// UseNormalPS.geom

// SetParameter GL_GEOMETRY_INPUT_TYPE_EXT=GL_POINTS
// SetParameter GL_GEOMETRY_OUTPUT_TYPE_EXT=GL_TRIANGLE_STRIP
// SetParameter GL_GEOMETRY_VERTICES_OUT_EXT=4

#version 130
#extension GL_EXT_geometry_shader4 : enable
#extension GL_EXT_gpu_shader4 : enable

//uniform float aspectAngleFactor; 

in vec3 positionIn[];
in int  subIndexIn[];
in float pointSizeIn[];

in vec3 viewVecIn[];
in vec3 normalIn[];
in vec3 binormalIn[];
in vec3 tangentIn[];

out vec3 position;
out vec3 viewVec;
out vec3 normal;
out vec3 binormal;
out vec3 tangent;

void main(void){

    vec4 l_tangent = vec4(tangentIn[0], 0);
    vec4 l_binormal = vec4(binormalIn[0], 0);

    vec4 offset[4] = vec4[]((-l_tangent-l_binormal), (+l_tangent-l_binormal),
			    (-l_tangent+l_binormal), (+l_tangent+l_binormal));

    /*
    vec4 offset[4] = vec4[]
	(vec4(-1, -1, 0,0), vec4(1, -1, 0,0), vec4(-1, 1, 0,0), vec4(1, 1, 0,0));
    */

    vec4 texCoord[4];
    int subIndex = subIndexIn[0];

    if (subIndex == 0){
	texCoord[0] = vec4(0, 0, 0 ,0);
	texCoord[1] = vec4(1, 0, 0, 0);
	texCoord[2] = vec4(0, 1, 0, 0);
	texCoord[3] = vec4(1, 1, 0, 0);
    }
    else{
	int atrasGrid = subIndex / 256;
	if (atrasGrid == 0) atrasGrid = 1;
	subIndex = subIndex % 256;
	
	float atrasWidth = 1.0 / atrasGrid;
	float atrasHeight = 1.0 / atrasGrid;
	
	int offsetX = subIndex % atrasGrid;
	int offsetY = subIndex / atrasGrid;
	texCoord[0] = vec4(offsetX * atrasWidth , offsetY * atrasHeight, 0, 0);
	texCoord[1] = vec4((offsetX + 1) * atrasWidth, offsetY * atrasHeight, 0, 0);
	texCoord[2] = vec4(offsetX * atrasWidth, (offsetY + 1) * atrasHeight, 0, 0);
	texCoord[3] = vec4((offsetX + 1) * atrasWidth, (offsetY + 1) * atrasHeight, 0, 0);
    }
   
    float psize = pointSizeIn[0];// * aspectAngleFactor;

    for(int i = 0; i < gl_VerticesIn; i++){
	viewVec = viewVecIn[i];
	normal = gl_NormalMatrix * normalIn[i];
	tangent = gl_NormalMatrix * tangentIn[i];
	binormal = cross(normal, tangent);

	gl_FrontColor = gl_FrontColorIn[i];
	gl_TexCoord[0] = texCoord[0];
	gl_Position = gl_ModelViewProjectionMatrix * (gl_PositionIn[i] + offset[0] * psize);
	position = positionIn[i] + vec3(offset[0]*psize);
	EmitVertex();
	gl_TexCoord[0] = texCoord[1];
	gl_Position = gl_ModelViewProjectionMatrix * (gl_PositionIn[i] + offset[1] * psize);
	position = positionIn[i] + vec3(offset[1]*psize);
	EmitVertex();
	gl_TexCoord[0] = texCoord[2];
	gl_Position = gl_ModelViewProjectionMatrix * (gl_PositionIn[i] + offset[2] * psize);
	position = positionIn[i] + vec3(offset[2]*psize);
	EmitVertex();
	gl_TexCoord[0] = texCoord[3];
	gl_Position = gl_ModelViewProjectionMatrix * (gl_PositionIn[i] + offset[3] * psize);
	position = positionIn[i] + vec3(offset[3]*psize);
	EmitVertex();
    }
    
    EndPrimitive();
}
